'use strict';

var ofxSerializer = require('ofx');

// Dependencies
var moment = require('./moment');

//  Functions
var csvAmount = function csvAmount(n) {
  return n > 0 ? `+${n.toFixed(2)}` : `${n.toFixed(2)}`;
};

var csvTransaction = function csvTransaction(t) {
  return `${moment(t.timestamp).format(moment.formats.default)}` + `,"${csvAmount(t.amount)}"` + `,"${t.description}"` + `,"${csvAmount(t.balance)}"\r\n`;
};

var qifTransaction = function qifTransaction(transaction) {
  var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';

  var format = void 0;
  switch (type) {
    case 'aus':
      format = moment.formats.aus;
      break;
    case 'us':
      format = moment.formats.us;
      break;
    default:
      format = moment.formats.default;
      break;
  }
  var lline = type === 'default' ? `L${transaction.amount >= 0 ? 'DEP' : 'DEBIT'}\r\n` : '';
  return `D${moment(transaction.timestamp).format(format)}\r\n` + `T${transaction.amount.toFixed(2)}\r\n` + `P${transaction.description}\r\n${lline}^\r\n`;
};

//  Transactions Serialization
var csv = function csv(transactions) {
  return transactions.map(csvTransaction).join('');
};
var qif = function qif(transactions) {
  var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'default';
  return `!Type:Bank\r\n${transactions.map(function (transaction) {
    return qifTransaction(transaction, type);
  }).join('')}`;
};
var ofx = function ofx(transactions, account, from, to) {
  var current = moment().format('YYYYMMDDHHmmss');
  var header = {
    OFXHEADER: '100',
    DATA: 'OFXSGML',
    VERSION: '102',
    SECURITY: 'NONE',
    ENCODING: 'USASCII',
    CHARSET: '1252',
    COMPRESSION: 'NONE',
    OLDFILEUID: 'NONE',
    NEWFILEUID: 'NONE'
  };

  var body = {
    SIGNONMSGSRSV1: {
      SONRS: {
        STATUS: {
          CODE: 0,
          SEVERITY: 'INFO'
        },
        DTSERVER: current,
        LANGUAGE: 'ENG'
      }
    }
  };

  //  BANKTRANLIST
  var translist = {
    DTSTART: moment(from, moment.formats.default).format('YYYYMMDD000000'),
    DTEND: moment(to, moment.formats.default).format('YYYYMMDD000000'),
    STMTTRN: transactions.map(function (t) {
      return {
        TRNTYPE: t.amount > 0 ? 'CREDIT' : 'DEBIT',
        DTPOSTED: moment(t.timestamp).format('YYYYMMDD'),
        DTUSER: moment(t.timestamp).format('YYYYMMDD'),
        TRNAMT: t.amount.toFixed(2),
        //  use timestamp as FITID if necessary to fix the OFX missing FITID issue.
        FITID: t.receiptnumber || t.timestamp,
        MEMO: t.description.replace(';', '')
      };
    })
  };

  if (account.type === 'DDA') {
    body.BANKMSGSRSV1 = {
      STMTTRNRS: {
        TRNUID: 1,
        STATUS: {
          CODE: 0,
          SEVERITY: 'INFO'
        },
        STMTRS: {
          CURDEF: 'AUD',
          BANKACCTFROM: {
            BANKID: account.bsb,
            ACCTID: account.account,
            ACCTTYPE: 'SAVINGS'
          },
          BANKTRANLIST: translist,
          LEDGERBAL: {
            BALAMT: account.balance,
            DTASOF: current
          },
          AVAILBAL: {
            BALAMT: account.available,
            DTASOF: current
          }
        }
      }
    };
  } else {
    body.CREDITCARDMSGSRSV1 = {
      CCSTMTTRNRS: {
        TRNUID: 1,
        STATUS: {
          CODE: 0,
          SEVERITY: 'INFO'
        },
        CCSTMTRS: {
          CURDEF: 'AUD',
          CCACCTFROM: {
            ACCTID: account.number
          },
          BANKTRANLIST: translist,
          LEDGERBAL: {
            BALAMT: account.balance,
            DTASOF: current
          },
          AVAILBAL: {
            BALAMT: account.available,
            DTASOF: current
          }
        }
      }
    };
  }

  return ofxSerializer.serialize(header, body);
};

//  Exports
module.exports = {
  csv,
  qif,
  ofx
};